@@ -16,6 +16,7 @@ import jsonpickle
 import numpy
 import pandas
 import xarray as xr
+from dash import Dash
 
 from pyplan_core import cubepy
 from pyplan_core.classes.BaseNode import BaseNode
@@ -23,11 +24,13 @@ from pyplan_core.classes.evaluators.Evaluator import Evaluator
 from pyplan_core.classes.Intellisense import Intellisense
 from pyplan_core.classes.IOModule import IOModule
 from pyplan_core.classes.PyplanFunctions import PyplanFunctions, Selector
-from pyplan_core.classes.wizards import (CalculatedField, DataarrayFromPandas,
+from pyplan_core.classes.wizards import (CalculatedField, CreateIndex,
+                                         DataArrayFilter, DataarrayFromPandas,
                                          DataframeGroupby, DataframeIndex,
-                                         InputTable, SelectColumns, SelectRows,
-                                         sourcecsv, CreateIndex, IndexFromPandas, DataArrayFilter, EditIndex, RenameIndexItem)
-from pyplan_core.classes.ws.settings import ws_settings
+                                         EditIndex, IndexFromPandas,
+                                         InputTable, RenameIndexItem,
+                                         SelectColumns, SelectRows, sourcecsv)
+from pyplan_core.classes.ws.settings import NotLevels
 
 from .DefaultNodeFormats import default_formats
 
@@ -58,6 +61,7 @@ class Model(object):
         self.session_key = None
         self.debugMode = None
         self.WS = WSClass
+        self.ws = None
 
     # Props
 
@@ -82,6 +86,25 @@ class Model(object):
     def getPID(self):
         return os.getpid()
 
+    def getCompanyCode(self):
+        return self.company_code
+
+    def getSessionKey(self):
+        return self.session_key
+
+    def setSessionKeyAndCompanyCode(self, session_key, company_code):
+        if session_key != self.session_key or company_code != self.company_code:
+            self.setWs(company_code, session_key)
+        return self.session_key
+
+    def setWs(self, company_code, session_key):
+        if company_code is not None and session_key is not None and self.WS is not None:
+            self.company_code = company_code
+            self.session_key = session_key
+            self.ws = self.WS(company_code=company_code,
+                              session_key=session_key)
+        return self.ws
+
     def getDefaultNodeFormat(self, nodeClass):
         default_formats
         if nodeClass in default_formats:
@@ -115,14 +138,6 @@ class Model(object):
         self._scenarioDic = dict()
         self._wizard = None
 
-    def connectToWS(self, company_code, session_key):
-        # Connect to WS
-        self.company_code = company_code
-        self.session_key = session_key
-        if self.WS:
-            self.ws = self.WS(company_code=company_code,
-                              session_key=session_key)
-
     def createNode(self, identifier=None, nodeClass=None, moduleId=None, x=None, y=None, toObj=False, originalId=None):
         """Create new node"""
         newNode = BaseNode(model=self, identifier=identifier, nodeClass=nodeClass,
@@ -290,7 +305,8 @@ class Model(object):
             return result
         finally:
             if self.debugMode and self.ws:
-                self.ws.sendDebugInfo("endPreview", "", "endPreview")
+                self.ws.ws_node_debug_information(
+                    node="endPreview", title="", action="endPreview")
             self.debugMode = None
 
     def getCubeValues(self, query):
@@ -873,7 +889,7 @@ class Model(object):
         res = []
         for k, v in self.nodeDic.items():
             if getattr(v, prop) == value:
-                if(not v.system):
+                if not v.system:
                     res.append(self.nodeDic[k])
         return res
 
@@ -1115,7 +1131,7 @@ class Model(object):
         }
 
         for k, v in self.nodeDic.items():
-            if(not v.system):
+            if not v.system:
                 toSave['nodeList'].append(v.toObj())
 
         if fileName:
@@ -1157,11 +1173,12 @@ class Model(object):
                     f.close()
         return jsonpickle.encode(toSave)
 
-    def openModel(self, fileName=None, textModel=None):
+    def openModel(self, fileName=None, company_code=None, session_key=None, textModel=None):
         """Open model.
         If fileName is especified then open from fileName, else open from textModel text """
 
         self.release()
+        self.setWs(company_code, session_key)
 
         opened = {}
         if textModel:
@@ -1630,19 +1647,30 @@ class Model(object):
                                    ] = installed_lib['version']
 
             to_install = ''
+
+            # send a message telling we are about to check libraries
+            if self.ws:
+                self.ws.ws_openening_model(
+                    title=f'Checking python libraries used in model', progress=75)
+
             for lib in modelLibs:
                 if not lib['name'] in installed_libs_dic:
                     to_install += f" {lib['name']}=={lib['version']}"
-                # TODO: if exists, check version and send message via channels
 
             if to_install != '':
-                self._installLibrary(to_install)
+                self._installLibrary(to_install, fromOpenModel=True)
+            else:
+                if self.ws:
+                    self.ws.ws_openening_model(
+                        title=f'Libraries already installed', progress=95)
 
         except Exception as ex:
             print(f'Error checking libraries. {ex}')
-            # TODO: send to client via channels
+            if self.ws:
+                self.ws.ws_notification_message(message=str(
+                    ex), title='Error checking libraries', not_level=NotLevels.ERROR)
 
-    def _installLibrary(self, lib):
+    def _installLibrary(self, lib, fromOpenModel=False):
 
         cmd = f'pip install {lib}'
 
@@ -1658,7 +1686,6 @@ class Model(object):
                              stderr=subprocess.PIPE, universal_newlines=True)
         nn = 0
         while p.stdout is not None and nn < 1200:
-            # TODO: show feedback to ide using channels
             line = p.stdout.readline()
             if not line:
                 p.stdout.flush()
@@ -1669,6 +1696,10 @@ class Model(object):
             sleep(1)
             nn += 1
             self._currentInstallProgress.append(line.rstrip('\n'))
+            # only send a ws message if we are opening a model
+            if self.ws and fromOpenModel:
+                self.ws.ws_openening_model(
+                    title=line.rstrip('\n'), progress=85)
 
         importlib.invalidate_caches()
 
@@ -1733,8 +1764,8 @@ class Model(object):
             return _used_libraries
         except Exception as ex:
             if self.ws:
-                self.ws.sendMsg(str(ex), 'Error getting used libraries',
-                                not_level=ws_settings.NOTIFICATION_LEVEL_ERROR)
+                self.ws.ws_notification_message(message=str(ex), title='Error getting used libraries',
+                                                not_level=NotLevels.ERROR)
             return []
 
     def _check_import_function(self, _element):
@@ -1789,8 +1820,8 @@ class Model(object):
             return installed_libraries
         except Exception as ex:
             if self.ws:
-                self.ws.sendMsg(str(ex), 'Error checking installed libraries',
-                                not_level=ws_settings.NOTIFICATION_LEVEL_ERROR)
+                self.ws.ws_notification_message(message=str(ex), title='Error checking installed libraries',
+                                                not_level=NotLevels.ERROR)
             return {}
 
     def listInstalledLibraries(self):
@@ -1865,3 +1896,28 @@ class Model(object):
         if not p2.match(final_text[0]):
             final_text = 'a' + final_text[1:]
         return final_text
+
+    def dispatchDashApp(self, node_id, params):
+        """Dispath Dash Application"""
+        app = self.getNode(node_id).result
+        if not isinstance(app, Dash):
+            return b'<html><body><h5>The node must be a Dash application</h5><body><html>'
+        pathname_prefix = f'/api/dashapp/{node_id}/{self.getNode("pyplan_user").result["session_key"]}/'
+        if not app is None:
+            app.config.set_read_only([])
+            app.config['requests_pathname_prefix'] = pathname_prefix
+            app.config['routes_pathname_prefix'] = pathname_prefix
+
+            if pathname_prefix in params["path"]:
+                params["path"] = params["path"][len(pathname_prefix)-1:]
+
+            with app.server.test_request_context(**params):
+                app.server.preprocess_request()
+                response = None
+                try:
+                    response = app.server.full_dispatch_request()
+                except Exception as e:
+                    response = app.server.make_response(
+                        app.server.handle_exception(e))
+
+                return response.get_data()
