@@ -48,7 +48,9 @@ class BaseEvaluator(object):
 
             return json.dumps(res)
 
-    def evaluateNode(self, result, nodeDic, nodeId, dims=None, rows=None, columns=None, summaryBy="sum", bottomTotal=False, rightTotal=False, fromRow=0, toRow=0, hideEmpty=None):
+    def evaluateNode(self, result, nodeDic, nodeId, dims=None, rows=None, columns=None,
+                     summaryBy="sum", bottomTotal=False, rightTotal=False, fromRow=0,
+                     toRow=0, hideEmpty=None, rowOrder='original', columnOrder='original'):
         if isinstance(result, np.ndarray):
             return self.createResult(result.tolist(), node=nodeDic[nodeId])
         elif callable(result):  # is function
