@@ -6,7 +6,7 @@ import pandas as pd
 from pyplan_core.classes.evaluators.BaseEvaluator import BaseEvaluator
 from pyplan_core.classes.common.filterChoices import filterChoices
 from pyplan_core.classes.common.indexValuesReq import IndexValuesReq
-from pyplan_core.classes.ws.settings import ws_settings
+from pyplan_core.classes.ws.settings import NotLevels
 from pyplan_core.cubepy.cube import kindToString
 from pyplan_core import cubepy
 
@@ -128,8 +128,8 @@ class PandasEvaluator(BaseEvaluator):
                         dfResult = dfResult.loc[:, filter_expresion]
                 except Exception as ex:
                     try:
-                        nodeDic[nodeId].model.ws.sendMsg(str(ex), 'Error applying empty data filter',
-                                                         not_level=ws_settings.NOTIFICATION_LEVEL_ERROR)
+                        nodeDic[nodeId].model.ws.ws_notification_message(message=str(ex), title='Error applying empty data filter',
+                                                         not_level=NotLevels.ERROR)
                     except:
                         pass
 
