@@ -9,7 +9,7 @@ from pyplan_core.classes.evaluators.BaseEvaluator import BaseEvaluator
 from pyplan_core.classes.evaluators.PandasEvaluator import PandasEvaluator
 from pyplan_core.classes.common.filterChoices import filterChoices
 from pyplan_core.classes.common.indexValuesReq import IndexValuesReq
-from pyplan_core.classes.ws.settings import ws_settings
+from pyplan_core.classes.ws.settings import NotLevels
 
 
 class XArrayEvaluator(BaseEvaluator):
@@ -121,8 +121,8 @@ class XArrayEvaluator(BaseEvaluator):
                     tmp = tmp.where(filter_expression, drop=True)
             except Exception as ex:
                 try:
-                    nodeDic[nodeId].model.ws.sendMsg(str(ex), 'Error applying empty data filter',
-                                                     not_level=ws_settings.NOTIFICATION_LEVEL_ERROR)
+                    nodeDic[nodeId].model.ws.ws_notification_message(message=str(ex), title='Error applying empty data filter',
+                                                     not_level=NotLevels.ERROR)
                 except:
                     pass
         
@@ -614,7 +614,7 @@ class XArrayEvaluator(BaseEvaluator):
                                     pass
 
             # perform aggregate
-            dataArray.coords[levels[0]].values = coordValues
+            dataArray = dataArray.assign_coords({levels[0]: coordValues})
             _df = dataArray.to_series()
             _df = _df.groupby(list(dataArray.dims), sort=False).agg(sby)
             _da = _df.to_xarray()
